from .core import ask_openai, unroll_prompt_from_file
import subprocess
import tempfile
import os
import platform
import shutil
import shlex


def use_tree_command(directory):
    """
    Build a tree-like listing of the contents of a directory and return the output.
    This function prints exactly what the linux command "tree" prints.

    Args:
        directory (str): The directory to list.
    """

    def tree(dir_path, prefix=""):
        entries = sorted(os.listdir(dir_path))
        lines = []
        for index, entry in enumerate(entries):
            path = os.path.join(dir_path, entry)
            # Decide which connector to use depending on the position
            if index == len(entries) - 1:
                connector = "└── "
                new_prefix = prefix + "    "
            else:
                connector = "├── "
                new_prefix = prefix + "│   "
            lines.append(prefix + connector + entry)
            if os.path.isdir(path):
                lines.extend(tree(path, new_prefix))
        return lines

    # Use the directory's basename as the root name (like tree does)
    root_name = os.path.basename(os.path.abspath(directory))
    output_lines = [root_name]
    output_lines.extend(tree(directory))

    # Now count the total directories and files
    total_dirs = total_files = 0
    for root, dirs, files in os.walk(directory):
        total_dirs += len(dirs)
        total_files += len(files)
    output_lines.append("")
    output_lines.append(f"{total_dirs} directories, {total_files} files")

    return "\n".join(output_lines)


def execute_bash_command(bash_code, directory):
    """
    Execute provided bash code in a temporary script file.
    On POSIX systems, this uses bash.
    On Windows, it first checks for bash availability (e.g. Git Bash, WSL)
    and if not found falls back to the default shell.

    Args:
        bash_code (str): The bash code to execute.
        directory (str): The working directory for execution.
    """
    # Create a temporary file for the script (outside the target directory)
    with tempfile.NamedTemporaryFile(mode="w", suffix=".sh", delete=False) as temp_file:
        temp_file.write(bash_code)
        temp_file.flush()
        temp_filename = temp_file.name

    try:
        # On POSIX systems, make the script executable.
        if os.name != "nt":
            os.chmod(temp_filename, 0o755)

        # Determine which shell to use.
        if os.name == "nt":
            # On Windows, try to locate bash first.
            bash_path = shutil.which("bash")
            if bash_path:
                command = [bash_path, temp_filename]
                result = subprocess.run(
                    command, cwd=directory, capture_output=True, text=True
                )
            else:
                # Fallback: use the default shell via shell=True.
                # Note: The code is written as bash, so execution may be affected.
                result = subprocess.run(
                    temp_filename,
                    cwd=directory,
                    capture_output=True,
                    text=True,
                    shell=True,
                )
        else:
            # On POSIX, use bash.
            result = subprocess.run(
                ["bash", temp_filename], cwd=directory, capture_output=True, text=True
            )
    finally:
        # Clean up the temporary file.
        os.remove(temp_filename)

    return result.stdout


def refactor_code(replace_map, directory):
    refactor_prompt = unroll_prompt_from_file("Refactor.txt", unroll=True)

    for key, value in replace_map.items():
        refactor_prompt = refactor_prompt.replace(f"[{key}]", value)

    bash_file_code = ask_openai(refactor_prompt, "high")

    command_output = execute_bash_command(bash_file_code, directory)

    return bash_file_code, command_output


def summarize_work_done(replace_map, directory):
    summary_prompt = unroll_prompt_from_file("RefactorSummary.txt", unroll=True)

    for key, value in replace_map.items():
        summary_prompt = summary_prompt.replace(f"[{key}]", value)

    summary = ask_openai(summary_prompt, "high")

    return summary


def checkpoint_next_action(replace_map, directory):
    checkpoint_prompt = unroll_prompt_from_file(
        "CheckpointerRedirecter.txt", unroll=True
    )

    for key, value in replace_map.items():
        checkpoint_prompt = checkpoint_prompt.replace(f"[{key}]", value)

    result = ask_openai(checkpoint_prompt, "high")

    return result


def code_test_command(test_command, directory):
    """
    Execute a test command in the specified directory.
    Uses OS detection to run the command appropriately.

    Args:
        test_command (str): The command to run.
        directory (str): The working directory for execution.
    """
    try:
        if os.name == "nt":
            # On Windows, use shell=True to process the command line properly.
            result = subprocess.run(
                test_command, cwd=directory, capture_output=True, text=True, shell=True
            )
        else:
            # On POSIX systems, split the command into args.
            args = shlex.split(test_command)
            result = subprocess.run(args, cwd=directory, capture_output=True, text=True)
    except Exception as e:
        return f"An error occurred while executing the test command: {e}"

    if result.stdout or result.stderr:
        return (
            f"STDOUT:\n{result.stdout}\n"
            f"STDERR:\n{result.stderr}\n"
            f"Exit code: {result.returncode}"
        )
    else:
        return f"Program exited with code {result.returncode}"


def execute_major_coding_task(task_instruction, directory, test_command):
    """
    Execute a major coding task.

    Args:
        task_instruction (str): The task instruction.
        directory (str): The directory where the task will be executed.
        test_command (str): The command used to test the code.
    """

    replace_map = {
        "TASK_INSTRUCTION_PLACEHOLDER": task_instruction,
        "TREE_COMMAND_PLACEHOLDER": use_tree_command(directory),
        "EXTRACTED_INFORMATION_PLACEHOLDER": "No extracted information yet",
        "SUMMARY_OF_WORK_DONE_PLACEHOLDER": "No work done yet, this is the first iteration",
        "TEST_OUTPUT_COMMAND": "No command was executed to test the code",
        "TEST_COMMAND": test_command,
    }

    while True:
        response = checkpoint_next_action(replace_map, directory)

        if response == "refactor":
            bash_file_code, command_output = refactor_code(replace_map, directory)
            replace_map["BASH_SCRIPT_PLACEHOLDER"] = bash_file_code
            summary_description = summarize_work_done(replace_map, directory)
            replace_map["SUMMARY_OF_WORK_DONE_PLACEHOLDER"] = summary_description
            replace_map["TEST_OUTPUT_COMMAND"] = code_test_command(
                test_command, directory
            )
            replace_map["TREE_COMMAND_PLACEHOLDER"] = use_tree_command(directory)

        elif response == "finish":
            return replace_map
